Class: RBS::Test::TypeCheck
- Inherits:
-
Object
- Object
- RBS::Test::TypeCheck
- Defined in:
- lib/rbs/test/type_check.rb,
sig/test/type_check.rbs
Constant Summary collapse
- DEFAULT_SAMPLE_SIZE =
100
Instance Attribute Summary collapse
-
#builder ⇒ Object
readonly
Returns the value of attribute builder.
-
#class_class ⇒ Object
readonly
Returns the value of attribute class_class.
-
#instance_class ⇒ Object
readonly
Returns the value of attribute instance_class.
-
#sample_size ⇒ Object
readonly
Returns the value of attribute sample_size.
-
#self_class ⇒ Object
readonly
Returns the value of attribute self_class.
-
#unchecked_classes ⇒ Object
readonly
Returns the value of attribute unchecked_classes.
Instance Method Summary collapse
- #args(method_name, method_type, fun, call, errors, type_error:, argument_error:) ⇒ Object
- #each_sample(array, &block) ⇒ Object
- #get_class(type_name) ⇒ Object
-
#initialize(self_class:, builder:, sample_size:, unchecked_classes:, instance_class: Object, class_class: Module) ⇒ TypeCheck
constructor
A new instance of TypeCheck.
- #is_double?(value) ⇒ Boolean
- #keyword?(value) ⇒ Boolean
-
#method_call(method_name, method_type, call, errors:, annotations: []) ⇒ Array[Errors::t]
Confirm if given
CallTraceis compatible withMethodType. - #overloaded_call(method, method_name, call, errors:) ⇒ Object
- #return(method_name, method_type, fun, call, errors, return_error:, annotations: []) ⇒ Object
-
#value(val, type) ⇒ Boolean
Test if given
valueis compatible to type. - #zip_args(args, fun, &block) ⇒ Object
- #zip_keyword_args(hash, fun) ⇒ Object
Constructor Details
#initialize(self_class:, builder:, sample_size:, unchecked_classes:, instance_class: Object, class_class: Module) ⇒ TypeCheck
Returns a new instance of TypeCheck.
15 16 17 18 19 20 21 22 |
# File 'lib/rbs/test/type_check.rb', line 15 def initialize(self_class:, builder:, sample_size:, unchecked_classes:, instance_class: Object, class_class: Module) @self_class = self_class @instance_class = instance_class @class_class = class_class @builder = builder @sample_size = sample_size @unchecked_classes = unchecked_classes.uniq end |
Instance Attribute Details
#builder ⇒ Object (readonly)
Returns the value of attribute builder.
7 8 9 |
# File 'lib/rbs/test/type_check.rb', line 7 def builder @builder end |
#class_class ⇒ Object (readonly)
Returns the value of attribute class_class.
11 12 13 |
# File 'lib/rbs/test/type_check.rb', line 11 def class_class @class_class end |
#instance_class ⇒ Object (readonly)
Returns the value of attribute instance_class.
10 11 12 |
# File 'lib/rbs/test/type_check.rb', line 10 def instance_class @instance_class end |
#sample_size ⇒ Object (readonly)
Returns the value of attribute sample_size.
8 9 10 |
# File 'lib/rbs/test/type_check.rb', line 8 def sample_size @sample_size end |
#self_class ⇒ Object (readonly)
Returns the value of attribute self_class.
6 7 8 |
# File 'lib/rbs/test/type_check.rb', line 6 def self_class @self_class end |
#unchecked_classes ⇒ Object (readonly)
Returns the value of attribute unchecked_classes.
9 10 11 |
# File 'lib/rbs/test/type_check.rb', line 9 def unchecked_classes @unchecked_classes end |
Instance Method Details
#args(method_name, method_type, fun, call, errors, type_error:, argument_error:) ⇒ Object
91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 |
# File 'lib/rbs/test/type_check.rb', line 91 def args(method_name, method_type, fun, call, errors, type_error:, argument_error:) test = zip_args(call.arguments, fun) do |val, param| unless self.value(val, param.type) errors << type_error.new(klass: self_class, method_name: method_name, method_type: method_type, param: param, value: val) end end unless test errors << argument_error.new(klass: self_class, method_name: method_name, method_type: method_type) end end |
#each_sample(array, &block) ⇒ Object
209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 |
# File 'lib/rbs/test/type_check.rb', line 209 def each_sample(array, &block) if block if sample_size && array.size > sample_size if sample_size > 0 size = array.size sample_size.times do yield array[rand(size)] end end else array.each(&block) end else enum_for :each_sample, array end end |
#get_class(type_name) ⇒ Object
226 227 228 229 230 |
# File 'lib/rbs/test/type_check.rb', line 226 def get_class(type_name) Object.const_get(type_name.to_s) rescue NameError nil end |
#is_double?(value) ⇒ Boolean
232 233 234 235 236 |
# File 'lib/rbs/test/type_check.rb', line 232 def is_double?(value) unchecked_classes.any? { |unchecked_class| Test.call(value, IS_AP, Object.const_get(unchecked_class))} rescue NameError false end |
#keyword?(value) ⇒ Boolean
152 153 154 |
# File 'lib/rbs/test/type_check.rb', line 152 def keyword?(value) Hash === value && value.each_key.all?(Symbol) end |
#method_call(method_name, method_type, call, errors:, annotations: []) ⇒ Array[Errors::t]
Confirm if given CallTrace is compatible with MethodType
Returns an array with detected errors.
10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 |
# File 'sig/test/type_check.rbs', line 10 def method_call(method_name, method_type, call, errors:, annotations: []) return errors if method_type.type.is_a?(Types::UntypedFunction) args(method_name, method_type, method_type.type, call.method_call, errors, type_error: Errors::ArgumentTypeError, argument_error: Errors::ArgumentError) self.return(method_name, method_type, method_type.type, call.method_call, errors, return_error: Errors::ReturnTypeError, annotations:) if method_type.block case when !call.block_calls.empty? call.block_calls.each do |block_call| args(method_name, method_type, method_type.block.type, block_call, errors, type_error: Errors::BlockArgumentTypeError, argument_error: Errors::BlockArgumentError) self.return(method_name, method_type, method_type.block.type, block_call, errors, return_error: Errors::BlockReturnTypeError) end when !call.block_given # Block is not given if method_type.block.required errors << Errors::MissingBlockError.new(klass: self_class, method_name: method_name, method_type: method_type) end else # Block is given, but not yielded end else if call.block_given errors << Errors::UnexpectedBlockError.new(klass: self_class, method_name: method_name, method_type: method_type) end end errors end |
#overloaded_call(method, method_name, call, errors:) ⇒ Object
24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 |
# File 'lib/rbs/test/type_check.rb', line 24 def overloaded_call(method, method_name, call, errors:) es = method.defs.map do |type_def| es = method_call(method_name, type_def.type, call, errors: [], annotations: type_def.annotations) if es.empty? return errors else es end end if es.size == 1 errors.push(*es[0]) else error = Errors::UnresolvedOverloadingError.new( klass: self_class, method_name: method_name, method_types: method.method_types ) RBS.logger.warn do tag = Errors.method_tag(error) = +"#{tag} UnresolvedOverloadingError " << method.method_types.zip(es).map do |method_type, es| msg = +"method_type=`#{method_type}`" details = es.map do |e| "\"#{Errors.to_string(e).sub("#{tag} ", "") }\"" end.join(', ') msg << " details=[#{details}]" end.join(', ') end errors << error end errors end |
#return(method_name, method_type, fun, call, errors, return_error:, annotations: []) ⇒ Object
109 110 111 112 113 114 115 116 117 118 119 120 121 |
# File 'lib/rbs/test/type_check.rb', line 109 def return(method_name, method_type, fun, call, errors, return_error:, annotations: []) if call.return? return if Test.call(call.return_value, IS_AP, NilClass) && annotations.find { |a| a.string == "implicitly-returns-nil" } unless value(call.return_value, fun.return_type) errors << return_error.new(klass: self_class, method_name: method_name, method_type: method_type, type: fun.return_type, value: call.return_value) end end end |
#value(val, type) ⇒ Boolean
Test if given value is compatible to type
Returns true if the value has the type.
16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 |
# File 'sig/test/type_check.rbs', line 16 def value(val, type) if is_double?(val) RBS.logger.info("A double (#{val.inspect}) is detected!") return true end case type when Types::Bases::Any true when Types::Bases::Bool Test.call(val, IS_AP, TrueClass) || Test.call(val, IS_AP, FalseClass) when Types::Bases::Top true when Types::Bases::Bottom false when Types::Bases::Void true when Types::Bases::Self Test.call(val, IS_AP, self_class) when Types::Bases::Nil Test.call(val, IS_AP, ::NilClass) when Types::Bases::Class Test.call(val, IS_AP, class_class) when Types::Bases::Instance Test.call(val, IS_AP, instance_class) when Types::ClassInstance klass = get_class(type.name) or return false if params = builder.env.normalized_module_class_entry(type.name.absolute!)&.type_params args = AST::TypeParam.normalize_args(params, type.args) if args.size != params.size return false end unless args == type.args type = Types::ClassInstance.new(name: type.name, args: args, location: type.location) end end case when klass == ::Array Test.call(val, IS_AP, klass) && each_sample(val).all? {|v| value(v, type.args[0]) } when klass == ::Hash Test.call(val, IS_AP, klass) && each_sample(val.keys).all? do |key| value(key, type.args[0]) && value(val[key], type.args[1]) end when klass == ::Range Test.call(val, IS_AP, klass) && value(val.begin, type.args[0]) && value(val.end, type.args[0]) when klass == ::Enumerator if Test.call(val, IS_AP, klass) case val.size when Float::INFINITY values = [] ret = self val.lazy.take(10).each do |*args| values << args nil end when nil values = [] count = 10_000 ret = val.each do |*args| count -= 1 values << args break if count <= 0 end if count == 0 ret = self end else values = [] ret = val.each do |*args| values << args nil end end value_check = values.empty? || each_sample(values).all? do |v| if v.size == 1 # Only one block argument. value(v[0], type.args[0]) || value(v, type.args[0]) else value(v, type.args[0]) end end return_check = if ret.equal?(self) type.args[1].is_a?(Types::Bases::Bottom) else value(ret, type.args[1]) end value_check && return_check end else Test.call(val, IS_AP, klass) end when Types::ClassSingleton klass = get_class(type.name) or return false singleton_class = begin klass.singleton_class rescue TypeError return false end Test.call(val, IS_AP, singleton_class) when Types::Interface if (definition = builder.build_interface(type.name.absolute!)) definition.methods.each.all? do |method_name, method| next false unless Test.call(val, RESPOND_TOP, method_name) meth = Test.call(val, METHOD, method_name) method.defs.all? do |type_def| type_def.member.overloads.all? do |overload| callable_argument?(meth.parameters, overload.method_type) end end end end when Types::Variable true when Types::Literal begin type.literal == val rescue NoMethodError raise if defined?(val.==) false end when Types::Union type.types.any? {|type| value(val, type) } when Types::Intersection type.types.all? {|type| value(val, type) } when Types::Optional Test.call(val, IS_AP, ::NilClass) || value(val, type.type) when Types::Alias value(val, builder.(type.name.absolute!, type.args)) when Types::Tuple Test.call(val, IS_AP, ::Array) && type.types.length == val.length && type.types.map.with_index {|ty, index| value(val[index], ty) }.all? when Types::Record Test::call(val, IS_AP, ::Hash) && type.fields.map {|key, type| value(val[key], type) }.all? when Types::Proc Test::call(val, IS_AP, ::Proc) else false end end |
#zip_args(args, fun, &block) ⇒ Object
156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 |
# File 'lib/rbs/test/type_check.rb', line 156 def zip_args(args, fun, &block) return true if fun.is_a?(Types::UntypedFunction) case when args.empty? if fun.required_positionals.empty? && fun.trailing_positionals.empty? && fun.required_keywords.empty? true else false end when !fun.required_positionals.empty? yield_self do param, fun_ = fun.drop_head yield(args.first, param) zip_args(args.drop(1), fun_, &block) end when fun.has_keyword? yield_self do hash = args.last if keyword?(hash) zip_keyword_args(hash, fun, &block) && zip_args(args.take(args.size - 1), fun.update(required_keywords: {}, optional_keywords: {}, rest_keywords: nil), &block) else fun.required_keywords.empty? && zip_args(args, fun.update(required_keywords: {}, optional_keywords: {}, rest_keywords: nil), &block) end end when !fun.trailing_positionals.empty? yield_self do param, fun_ = fun.drop_tail yield(args.last, param) zip_args(args.take(args.size - 1), fun_, &block) end when !fun.optional_positionals.empty? yield_self do param, fun_ = fun.drop_head yield(args.first, param) zip_args(args.drop(1), fun_, &block) end when fun.rest_positionals yield_self do yield(args.first, fun.rest_positionals) zip_args(args.drop(1), fun, &block) end else false end end |
#zip_keyword_args(hash, fun) ⇒ Object
123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 |
# File 'lib/rbs/test/type_check.rb', line 123 def zip_keyword_args(hash, fun) fun.required_keywords.each do |name, param| if hash.key?(name) yield(hash[name], param) else return false end end fun.optional_keywords.each do |name, param| if hash.key?(name) yield(hash[name], param) end end hash.each do |name, value| next if fun.required_keywords.key?(name) next if fun.optional_keywords.key?(name) if fun.rest_keywords yield value, fun.rest_keywords else return false end end true end |