Class: RBS::AST::TypeParam
- Inherits:
-
Object
- Object
- RBS::AST::TypeParam
- Includes:
- _ToJson
- Defined in:
- lib/rbs/ast/type_param.rb,
sig/type_param.rbs
Instance Attribute Summary collapse
-
#default_type ⇒ Types::t?
readonly
Returns the value of attribute default_type.
-
#location ⇒ loc?
readonly
Returns the value of attribute location.
-
#lower_bound_type ⇒ Types::t?
readonly
Returns the value of attribute lower_bound_type.
-
#name ⇒ Symbol
readonly
Returns the value of attribute name.
-
#upper_bound_type ⇒ Types::t?
readonly
Returns the value of attribute upper_bound_type.
-
#variance ⇒ variance
readonly
Returns the value of attribute variance.
Class Method Summary collapse
-
.application(params, args) ⇒ Substitution?
Returns an application with respect to type params' default.
-
.normalize_args(params, args) ⇒ Array[Types::t]
Returns an array of type args, that fills omitted types with the defaults.
-
.rename(params, new_names:) ⇒ Array[TypeParam]
Rename type parameter name.
-
.resolve_variables(params) ⇒ void
Helper function to resolve class instance types to type variables.
- .subst_var(vars, type) ⇒ Types::t
-
.validate(type_params) ⇒ Array[TypeParam]?
Validates TypeParams if it refers another optional type params.
Instance Method Summary collapse
- #==(other) ⇒ Boolean (also: #eql?)
- #hash ⇒ Integer
-
#initialize(name:, variance:, upper_bound:, lower_bound:, location:, default_type: nil, unchecked: false) ⇒ TypeParam
constructor
A new instance of TypeParam.
- #lower_bound ⇒ Object
- #map_type {|arg0| ... } ⇒ TypeParam
- #to_json(state = JSON::State.new) ⇒ Object
- #to_s ⇒ String
- #unchecked!(value = true) ⇒ self
- #unchecked? ⇒ Boolean
- #upper_bound ⇒ Object
Constructor Details
#initialize(name:, variance:, upper_bound:, lower_bound:, location:, default_type: nil, unchecked: false) ⇒ TypeParam
Returns a new instance of TypeParam.
8 9 10 11 12 13 14 15 16 |
# File 'lib/rbs/ast/type_param.rb', line 8 def initialize(name:, variance:, upper_bound:, lower_bound:, location:, default_type: nil, unchecked: false) @name = name @variance = variance @upper_bound_type = upper_bound @lower_bound_type = lower_bound @location = location @default_type = default_type @unchecked = unchecked end |
Instance Attribute Details
#default_type ⇒ Types::t? (readonly)
Returns the value of attribute default_type.
6 7 8 |
# File 'lib/rbs/ast/type_param.rb', line 6 def default_type @default_type end |
#location ⇒ loc? (readonly)
Returns the value of attribute location.
6 7 8 |
# File 'lib/rbs/ast/type_param.rb', line 6 def location @location end |
#lower_bound_type ⇒ Types::t? (readonly)
Returns the value of attribute lower_bound_type.
6 7 8 |
# File 'lib/rbs/ast/type_param.rb', line 6 def lower_bound_type @lower_bound_type end |
#name ⇒ Symbol (readonly)
Returns the value of attribute name.
6 7 8 |
# File 'lib/rbs/ast/type_param.rb', line 6 def name @name end |
#upper_bound_type ⇒ Types::t? (readonly)
Returns the value of attribute upper_bound_type.
6 7 8 |
# File 'lib/rbs/ast/type_param.rb', line 6 def upper_bound_type @upper_bound_type end |
#variance ⇒ variance (readonly)
Returns the value of attribute variance.
6 7 8 |
# File 'lib/rbs/ast/type_param.rb', line 6 def variance @variance end |
Class Method Details
.application(params, args) ⇒ Substitution?
Returns an application with respect to type params' default
92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 |
# File 'sig/type_param.rbs', line 92 def self.application(params, args) if params.empty? return nil end optional_params, required_params = params.partition {|param| param.default_type } param_subst = Substitution.new() app_subst = Substitution.new() required_params.zip(args.take(required_params.size)).each do |param, arg| arg ||= Types::Bases::Any.new(location: nil) param_subst.add(from: param.name, to: arg) app_subst.add(from: param.name, to: arg) end optional_params.each do |param| param_subst.add(from: param.name, to: Types::Bases::Any.new(location: nil)) end optional_params.zip(args.drop(required_params.size)).each do |param, arg| if arg app_subst.add(from: param.name, to: arg) else param.default_type or raise app_subst.add(from: param.name, to: param.default_type.sub(param_subst)) end end app_subst end |
.normalize_args(params, args) ⇒ Array[Types::t]
Returns an array of type args, that fills omitted types with the defaults
interface _Foo[T, S = untyped]
end
Normalizing type args with _Foo works as following:
_Foo[String] # => _Foo[String, untyped]
_Foo[String, Integer] # => _Foo[String, Integer]
_Foo # => _Foo (Omitting missing args)
_Foo[String, Integer, untyped] # => _Foo[String, Integer, untyped] (Keeping extra args)
Note that it allows invalid arities, returning the args immediately.
112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 |
# File 'sig/type_param.rbs', line 112 def self.normalize_args(params, args) app = application(params, args) or return args min_count = params.count { _1.default_type.nil? } unless min_count <= args.size && args.size <= params.size return args end params.zip(args).filter_map do |param, arg| if arg arg else if param.default_type param.default_type.sub(app) else Types::Bases::Any.new(location: nil) end end end end |
.rename(params, new_names:) ⇒ Array[TypeParam]
Rename type parameter name.
The renaming cannot be done separately because a set of TypeParam decls may be mutual recursive.
Example:
- Renaming
A -> X, B -> Y - Input
[A, B < _Pushable[A]] - Result
[X, Y < _Pushable[X]]
79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 |
# File 'sig/type_param.rbs', line 79 def self.rename(params, new_names:) raise unless params.size == new_names.size subst = Substitution.build(new_names, Types::Variable.build(new_names)) params.map.with_index do |param, index| new_name = new_names[index] TypeParam.new( name: new_name, variance: param.variance, upper_bound: param.upper_bound_type&.map_type {|type| type.sub(subst) }, lower_bound: param.lower_bound_type&.map_type {|type| type.sub(subst) }, location: param.location, default_type: param.default_type&.map_type {|type| type.sub(subst) } ).unchecked!(param.unchecked?) end end |
.resolve_variables(params) ⇒ void
This method returns an undefined value.
Helper function to resolve class instance types to type variables.
We need this step because RBS language has an identical syntax for both unqualified class instance types and type variables.
String may be an instance of ::String class or type variable depending on the list of bound type variables.
So, we need second pass to parse the following generics parameter declaration.
class Foo[X < _Each[Y], Y]
# ^ We want this `Y` to be a type variable.
end
65 66 67 68 69 70 71 72 73 |
# File 'sig/type_param.rbs', line 65 def self.resolve_variables(params) return if params.empty? vars = Set.new(params.map(&:name)) params.map! do |param| param.map_type {|bound| _ = subst_var(vars, bound) } end end |
.subst_var(vars, type) ⇒ Types::t
102 103 104 105 106 107 108 109 110 111 112 |
# File 'lib/rbs/ast/type_param.rb', line 102 def self.subst_var(vars, type) case type when Types::ClassInstance namespace = type.name.namespace if namespace.relative? && namespace.empty? && vars.member?(type.name.name) return Types::Variable.new(name: type.name.name, location: type.location) end end type.map_type {|t| subst_var(vars, t) } end |
.validate(type_params) ⇒ Array[TypeParam]?
Validates TypeParams if it refers another optional type params
- Returns array of TypeParam objects that refers other optional type params
- Returns
nilif all type params are valid
88 89 90 91 92 93 94 95 96 97 98 99 100 101 |
# File 'sig/type_param.rbs', line 88 def self.validate(type_params) optionals = type_params.filter {|param| param.default_type } optional_param_names = optionals.map(&:name).sort optionals.filter! do |param| default_type = param.default_type or raise optional_param_names.any? { default_type.free_variables.include?(_1) } end unless optionals.empty? optionals end end |
Instance Method Details
#==(other) ⇒ Boolean Also known as: eql?
41 42 43 44 45 46 47 48 49 |
# File 'lib/rbs/ast/type_param.rb', line 41 def ==(other) other.is_a?(TypeParam) && other.name == name && other.variance == variance && other.upper_bound_type == upper_bound_type && other.lower_bound_type == lower_bound_type && other.default_type == default_type && other.unchecked? == unchecked? end |
#hash ⇒ Integer
53 54 55 |
# File 'lib/rbs/ast/type_param.rb', line 53 def hash self.class.hash ^ name.hash ^ variance.hash ^ upper_bound_type.hash ^ lower_bound_type.hash ^ unchecked?.hash ^ default_type.hash end |
#lower_bound ⇒ Object
25 26 27 28 29 30 |
# File 'lib/rbs/ast/type_param.rb', line 25 def lower_bound case lower_bound_type when Types::ClassInstance, Types::ClassSingleton, Types::Interface lower_bound_type end end |
#map_type {|arg0| ... } ⇒ TypeParam
69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 |
# File 'lib/rbs/ast/type_param.rb', line 69 def map_type(&block) if b = upper_bound_type _upper_bound_type = yield(b) end if b = lower_bound_type _lower_bound_type = yield(b) end if dt = default_type _default_type = yield(dt) end TypeParam.new( name: name, variance: variance, upper_bound: _upper_bound_type, lower_bound: _lower_bound_type, location: location, default_type: _default_type ).unchecked!(unchecked?) end |
#to_json(state = JSON::State.new) ⇒ Object
57 58 59 60 61 62 63 64 65 66 67 |
# File 'lib/rbs/ast/type_param.rb', line 57 def to_json(state = JSON::State.new) { name: name, variance: variance, unchecked: unchecked?, location: location, upper_bound: upper_bound_type, lower_bound: lower_bound_type, default_type: default_type }.to_json(state) end |
#to_s ⇒ String
133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 |
# File 'lib/rbs/ast/type_param.rb', line 133 def to_s s = +"" if unchecked? s << "unchecked " end case variance when :invariant # nop when :covariant s << "out " when :contravariant s << "in " end s << name.to_s if type = upper_bound_type s << " < #{type}" end if type = lower_bound_type s << " > #{type}" end if dt = default_type s << " = #{dt}" end s end |
#unchecked!(value = true) ⇒ self
32 33 34 35 |
# File 'lib/rbs/ast/type_param.rb', line 32 def unchecked!(value = true) @unchecked = value ? true : false self end |
#unchecked? ⇒ Boolean
37 38 39 |
# File 'lib/rbs/ast/type_param.rb', line 37 def unchecked? @unchecked end |
#upper_bound ⇒ Object
18 19 20 21 22 23 |
# File 'lib/rbs/ast/type_param.rb', line 18 def upper_bound case upper_bound_type when Types::ClassInstance, Types::ClassSingleton, Types::Interface upper_bound_type end end |